######################################################################################################
#									
# 	This R Program randomly generates dose-toxicity curves
#	according to the algorithm of Clertant and O'Quigley (2017)*
#
#	*Clertant, M., O'Quigley, J. (2017). Semiparametric dose-finding methods. 
#		Journal of the Royal Statistical Society - Series B (Statistical Methodology) 79, 1487-1508.
#
#######################################################################################################
# Pseudo-Uniform Curve Generation Function

# phi     -  Target DLT Rate
# J       -  Number of Dose Levels
# ncurves -  Number of curves to be generated

curve_gen <- function(phi,J,ncurves){
  
  dose.levels <- c(1:J)                 # Creates a vector of dose levels.
  
  curves_mat <- matrix(0,nrow = ncurves, ncol = J)    # This matrix will hold the resulting curves
  
  for(k in 1:ncurves){
  i=0
  
  mtd <- sample(dose.levels,1)          #Selects one dose level at random to be the mtd.
  M <- rbeta(n=1,max(J-mtd,.5),1)
  B <- phi + (1-phi)*M                  #Establishes an upperbound on toxicity probabilities.
  

  while(i == 0){
    curvetry <- sort(runif(n=J,min=0,max=B))
    mtdlevel <- which.min(abs(curvetry-phi))
  
    if(mtdlevel==mtd){i = i+1}                        # While loop breaks if generate a scenario using the upper bound where the mtd matches from above
  }
  curves_mat[k,] <- curvetry                        # Puts the accepted curve into the matrix
  
  }
  curves <- as.data.frame(curves_mat)               # Coerces the matrix of values to be returned as a data frame in the next statement
  return(curves)
  
}

a <- curve_gen(.25,6,200)
matplot(t(a),type="l",lty=1,col="black",ylim=c(0,1),xlab="Dose level",ylab="Probability of DLT",
		cex=1.5,cex.lab=1.5,cex.axis=1.5, cex.main=1.5, cex.sub=1.5)